package v1alpha1

import (
	networkingv1 "k8s.io/api/networking/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	cfg "sigs.k8s.io/controller-runtime/pkg/config/v1alpha1"
)

//+kubebuilder:object:root=true

// LibreConfig is the Schema for the libreconfigs API
type LibreConfig struct {
	metav1.TypeMeta `json:",inline"`

	// ControllerManagerConfigurationSpec returns the contfigurations for controllers
	cfg.ControllerManagerConfigurationSpec `json:",inline"`

	//+required
	KeycloakProviders []Provider `json:"keycloakProviders"`
	//+required
	MinIOProviders []MinIOProvider `json:"minioProviders"`
	//+required
	ViMbAdminProviders []Provider `json:"vimbadminProviders"`
}

type Provider struct {
	//+required
	Name string `json:"name"`
	//+required
	Endpoint string `json:"endpoint"`
	//+optional
	Insecure bool `json:"insecure,omitempty"`
	//+required
	Username string `json:"username"`
	//+required
	Password string `json:"password"`
}

type MinIOProvider struct {
	//+required
	Provider `json:",inline"`
	//+optional
	BucketDNS *MinIOBucketDNS `json:"bucketDNS,omitempty"`
}

type MinIOBucketDNS struct {
	//+requierd
	Namespace string `json:"namespace"`
	//+requierd
	ServiceBackend networkingv1.IngressServiceBackend `json:"serviceBackend"`
	//+optional
	Annotations map[string]string `json:"annotations,omitempty"`
}

func (config *LibreConfig) GetMinIOProvider(name string) *MinIOProvider {
	for _, p := range config.MinIOProviders {
		if p.Name == name {
			return &p
		}
	}
	return nil
}

func (config *LibreConfig) GetKeycloakProvider(name string) *Provider {
	for _, p := range config.KeycloakProviders {
		if p.Name == name {
			return &p
		}
	}
	return nil
}

func (config *LibreConfig) GetViMbAdminProvider(name string) *Provider {
	for _, p := range config.ViMbAdminProviders {
		if p.Name == name {
			return &p
		}
	}
	return nil
}

func init() {
	SchemeBuilder.Register(&LibreConfig{})
}
