package internal

import (
	"context"
	"fmt"

	"github.com/minio/madmin-go"
	"github.com/minio/minio-go/v7"
	"github.com/minio/minio-go/v7/pkg/credentials"
	configv1alpha1 "libre.sh/controller/apis/config/v1alpha1"
	corev1alpha1 "libre.sh/controller/apis/core/v1alpha1"
	"sigs.k8s.io/controller-runtime/pkg/client"
)

type MinioClient struct {
	*minio.Client
	Admin    *madmin.AdminClient
	Provider *configv1alpha1.MinIOProvider
}

func GetMinioClient(ctx context.Context, c client.Client, config *configv1alpha1.LibreConfig, bucket *corev1alpha1.Bucket) (MinioClient, error) {
	provider := config.GetMinIOProvider(bucket.Spec.Provider)
	if provider == nil {
		return MinioClient{}, fmt.Errorf("provider %s not found", bucket.Spec.Provider)
	}

	minioClient, err := minio.New(provider.Endpoint, &minio.Options{
		Creds:  credentials.NewStaticV4(provider.Username, provider.Password, ""),
		Secure: !provider.Insecure,
	})
	if err != nil {
		return MinioClient{}, err
	}
	minioAdminClient, err := madmin.New(provider.Endpoint, provider.Username, provider.Password, !provider.Insecure)
	if err != nil {
		return MinioClient{}, err
	}
	return MinioClient{
		Client:   minioClient,
		Admin:    minioAdminClient,
		Provider: provider,
	}, nil
}
